﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MicroBloggingAzure.Models;

namespace MicroBloggingAzure.Controllers
{
    public class IndexViewData
    {
        public IEnumerable<Message> Messages { get; set; }
        public IEnumerable<Channel> Channels { get; set; }
    }

    [HandleError]
    public class HomeController : Controller
    {
        public ActionResult Index(string channel, string author)
        {
            var svc = new Models.MicroData();
            ViewData["channels"] = svc.Channels;

            var q = from m in svc.Messages
                    where m.PartitionKey == Models.Message.MakeKey(channel, DateTime.UtcNow)
                    select m;
            if (!string.IsNullOrEmpty(author))
            {
                q = from m in q
                    where m.Author == author
                    select m;
            }

            ViewData["msg"] = q;

            return View();
        }
        public ActionResult Message()
        {
            return View();
        }

        [AcceptVerbs("POST")]
        public ActionResult PostMessage(string channel, string author, string msg)
        {
            var svc = new Models.MicroData();

            var q = from c in svc.Channels
                    where c.PartitionKey == channel
                    select c;

            if (q.FirstOrDefault() == null) throw new ArgumentException("Invalid channel");

            User u = (from auth in svc.Users
                     where auth.UserName == author
                     select auth).FirstOrDefault();
            
            if (u == null) //throw new ArgumentException("Invalid Author");
            {
                var user = new User();
                user.UserName = author;
                user.DateCreated = DateTime.UtcNow;
                user.PartitionKey = user.UserName;
                user.RowKey = string.Empty;
                svc.AddObject("Users", user);
                u = user;
            }

            var m = new Models.Message(channel, author, msg);
            svc.AddObject("Messages", m);

            svc.SaveChanges();

            return this.RedirectToAction("Index");
        }
        public ActionResult Channel()
        {
            var svc = new Models.MicroData();
            ViewData["channels"] = svc.Channels;

            return View();
        }
        [AcceptVerbs("POST")]
        public ActionResult CreateChannel(string name)
        {
            var svc = new Models.MicroData();
            Channel chan = (from c in svc.Channels
                    where c.ChannelName == name
                    select c).FirstOrDefault();
            

            if (chan != null) throw new ArgumentException("Channel Already Exists.");

            var ch = new Channel();
            ch.ChannelName = name;
            ch.DateCreated = DateTime.UtcNow;
            ch.PartitionKey = ch.ChannelName;
            ch.RowKey = string.Empty;

            svc.AddObject("Channels", ch);
            svc.SaveChanges();

            return this.RedirectToAction("Index");
        }
        public ActionResult About()
        {
            ViewData["Title"] = "About Page";

            return View();
        }

        
    }
}
