﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data.Services.Common;
using Microsoft.Samples.ServiceHosting.StorageClient;

namespace MicroBloggingAzure.Models
{
    [DataServiceKey("PartitionKey", "RowKey")]
    public class Channel
    {
        // the channel name
        public string PartitionKey { get; set; }
        public string RowKey { get; set; }

        public string ChannelName { get; set; }
        public DateTime DateCreated { get; set; }

        public Channel()
        {
        }

        public Channel(string name)
        {
            this.ChannelName = name;
            this.DateCreated = DateTime.UtcNow;
            this.PartitionKey = ChannelName;
            this.RowKey = string.Empty;
        }
    }

    [DataServiceKey("PartitionKey", "RowKey")]
    public class Message
    {
        // ParitionKey is channel name [+ day]
        public string PartitionKey { get; set; }
        // Date/Time when the message was created
        public string RowKey { get; set; }

        public string Text { get; set; }
        public string Author { get; set; }
        public string Channel { get; set; }
        public DateTime DatePosted { get; set; }
        public int Rating { get; set; }

        public Message()
        {
        }

        public Message(string channel, string author, string text)
        {
            this.Text = text;
            this.Author = author;
            this.Channel = channel;
            this.DatePosted = DateTime.UtcNow;

            this.PartitionKey = MakeKey(this.Channel, this.DatePosted);
            this.RowKey = System.Xml.XmlConvert.ToString(this.DatePosted);
        }

        public static string MakeKey(string channel, DateTime dt)
        {
            return channel + "@" + dt.ToString("yyyy-MM-dd");
        }
    }

    [DataServiceKey("PartitionKey", "RowKey")]
    public class User
    {
        // the username
        public string PartitionKey { get; set; }
        public string RowKey { get; set; }

        public string UserName { get; set; }
        public DateTime DateCreated { get; set; }


        public User()
        {
        }

        public User(string name)
        {
            this.DateCreated = DateTime.UtcNow;
            this.UserName = name;
            this.PartitionKey = UserName;
            this.RowKey = string.Empty;

        }
    }

    public class MicroData : TableStorageDataServiceContext
    {
        public IQueryable<Channel> Channels
        {
            get { return this.CreateQuery<Channel>("Channels"); }
        }
        public IQueryable<Message> Messages
        {
            get { return this.CreateQuery<Message>("Messages"); }
        }
        public IQueryable<User> Users
        {
            get { return this.CreateQuery<User>("Users"); }
        }
    }
}
