﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MicroBloggingOnPremise.Models;
using System.Data.Services.Client;

namespace MicroBloggingOnPremise.Controllers
{
    [HandleError]
    public class HomeController : Controller
    {
        public ActionResult Index(string channel, string author)
        {
            microblogEntities svc = new microblogEntities(new Uri("http://localhost:50396/MicroData.svc"));
            ViewData["channels"] = svc.Channels;

            int y = DateTime.Now.Year;
            int mm = DateTime.Now.Month;
            int d = DateTime.Now.Day;

            var q = from m in svc.Messages.Expand("Author").Expand("Channel")
                    where m.Channel.ChannelName == channel && (m.DatePosted.Year == y && m.DatePosted.Month == mm && m.DatePosted.Day == d)
                    select m;
            
            if (!string.IsNullOrEmpty(author))
            {
                q = from m in q
                    where m.Author.UserName == author
                    select m;
            }

            ViewData["msg"] = q;

            return View();
        }

        public ActionResult Message()
        {
            return View();
        }
        [AcceptVerbs("POST")]
        public ActionResult PostMessage(string channel, string author, string msg)
        {
            microblogEntities svc = new microblogEntities(new Uri("http://localhost:50396/MicroData.svc"));

            Channel chan = (from c in svc.Channels
                    where c.ChannelName == channel
                    select c).FirstOrDefault();
             

            if (chan == null) throw new ArgumentException("Invalid channel");

            User u = (from auth in svc.Users
                     where auth.UserName == author
                     select auth).FirstOrDefault();
            
            if (u == null) //throw new ArgumentException("Invalid Author");
            {
                //To simplify this example we will create a new user, if the user was null when we queried. 
                //It is possible that another client creates the same UserName before we call SaveChanges() 
                //and we end up with 2 Users with the same name
                var user = new User();
                user.UserName = author;
                user.DateCreated = DateTime.UtcNow;
                svc.AddToUsers(user);
                u = user;

            }


            var m = new Message();
            
            m.DatePosted = DateTime.UtcNow;
            m.Text = msg;
            svc.AddToMessages(m);
            svc.SetLink(m, "Channel", chan);
            svc.SetLink(m, "Author", u);

            try
            {
                svc.SaveChanges();
            }
            catch (Exception e)
            {
                throw (e);
            }
            return this.RedirectToAction("Index");
        }
        public ActionResult Channel()
        {
            microblogEntities svc = new microblogEntities(new Uri("http://localhost:50396/MicroData.svc"));
            ViewData["channels"] = svc.Channels;
            return View();
        }
        [AcceptVerbs("POST")]
        public ActionResult CreateChannel(string name)
        {
            microblogEntities svc = new microblogEntities(new Uri("http://localhost:50396/MicroData.svc"));
           
            var q = from c in svc.Channels
                    where c.ChannelName == name
                    select c;
            Channel chan = q.FirstOrDefault();

            if (chan != null) throw new ArgumentException("Channel Already Exists.");

            //To simplify this example we will create a new channel, if the channel            
            //was null when we queried. 
            //It is possible that another client creates the same channel 
            //before we call SaveChanges() and we end up with 2 Channels with the 
            //same name
            var ch = new Channel();
            ch.ChannelName = name;
            ch.DateCreated = DateTime.UtcNow;

            svc.AddToChannels(ch);
            svc.SaveChanges();

            return this.RedirectToAction("Index");
        }

        public ActionResult About()
        {
            return View();
        }
    }
}
